#Copyright (C)2022  Elias Kremer, full license (GNU AGPLv3) in LICENSE.txt
from .errorhandlers import *

class ReadConfig:
    def __init__(self, configpath, file = "project.conf"):
        from os import path
        self.configpath = f'{configpath}/config/{file}'
        self.config = []

        if path.exists(self.configpath):
            configfile = open(self.configpath, "r")
            for line in configfile:
                x = line.strip()
                if x[:1] != "#" and x[:1] != "": self.config.append(x)
            configfile.close()
        else: raise LoadError(f'Config file not found at {self.configpath}')

    def getconfig(self, type = None):

        if type == "all" or type == None: return self.config

        elif type == "dsbcredentials":
            return {"username": self.config[0], "password": self.config[1]}

        elif type == "mySQL":
            return {"host": self.config[2], "username": self.config[3], "password": self.config[4], "dbname": self.config[5]}

        elif type == "logmode":
            return self.config[6]

        # elif type == "ci": #Controll Interface
        #     return {"port": int(self.config[10]),
        #             "password": self.config[11]}

        # elif type == "li": #Live Interface
        #     return {"port": int(self.config[12]),
        #             "password": self.config[13]}

        else: raise LoadError(f'Unknown argument "{str(type)}" for function getconfig()')

class mySQL:
    # configs is the ReadConfig object from above
    def __init__(self, configs):
        self.init(configs)

    def init(self, configs):
         import mysql.connector as sql
         self.InterfaceError = sql.errors.InterfaceError
         self.OperationalError = sql.errors.OperationalError

         credentials = configs.getconfig("mySQL")

         try: self.connection = sql.connect(host=credentials["host"],
                                            user=credentials["username"],
                                            passwd=credentials["password"])
         except Exception as er: raise mySQLError(er) from None

         self.cursor = self.connection.cursor()
         self.cursor.execute("SHOW DATABASES;")
         res = self.cursor.fetchall()
         res = [x for sublist in res for x in sublist]
         if not credentials["dbname"] in res: self.cursor.execute(f'CREATE DATABASE {credentials["dbname"]};')
         self.cursor.execute(f'use {credentials["dbname"]};')
         return

    def sendquery(self, query):
        try:
            self.cursor.execute(query)
        except self.OperationalError:
            try:
                self.close()
                self.init()
                self.cursor.execute(query)
            except self.OperationalError as er:
                self.crashreport = er
                res = False

        try:
            res = self.cursor.fetchall()
            self.multiline = res
            res = [x for sublist in res for x in sublist]
        except self.InterfaceError as er:
            self.crashreport = er
            self.multiline = None
            res = False

        return res

    def commit(self):
        self.connection.commit()

    def close(self):
        self.connection.commit()
        self.cursor.close()
        self.connection.close()

def version():
    return 4.5 #Generalized version of 4.1



#Copyright (C)2022  Elias Kremer, full license (GNU AGPLv3) in LICENSE.txt
from .errorhandlers import *
from datetime import datetime, date
strptime = datetime.strptime

class MySQLLogger:
    def __init__(self, config_obj, path):
        from .load import ReadConfig, mySQL
        from .logger import Logger

        self.logging = Logger(config_obj = config_obj, path = path, short = "SQLL", name = "db.log")
        self.keyerrorlogging = Logger(config_obj = config_obj, path = path, name = "keyerror.log")
        self.mysql = mySQL(config_obj)
        self.stats = 0
        self.logging.append("DB ready...")

    def create_table(self, name = "people", col = [["name", "VARCHAR(10)"], ["age", "INTEGER"]]):
        if name == "currentdate": name = str(date.today()).replace("-","_")
        self.name = name
        self.col = col

        query = f'CREATE TABLE IF NOT EXISTS {name} ('
        for pair in col:
            query += f'{pair[0]} {pair[1]},'
        query = query[:-1] + ");"

        self.mysql.sendquery(query)
        self.logging.append(f'Created table {name}')
        return

    def write(self, beacon, col = "standart", name = "standart"):
        if col == "standart": col = self.col
        if name == "standart": name = self.name
        self.verify(beacon)
        try:
            query = f'INSERT INTO {name} VALUES ('
            for pair in col:
                if pair[1] == "DATETIME":
                    try: query += f'"{strptime(str(beacon[pair[0]]), "%d.%m.%Y %H:%M")}",'
                    except ValueError: query += f'"{strptime(str(beacon[pair[0]]), "%d.%m.%Y")}",'
                else:
                    query += f'"{str(beacon[pair[0]])}",'
            query = query[:-1] + ");"
        except KeyError as e:
            self.keyerrorlogging.append(e)
            return False
        except TypeError as e:
            self.keyerrorlogging.append(e)
            return False

        self.mysql.sendquery(query)
        self.mysql.commit()
        self.stats += 1
        return True

    def verify(self, beacon):
        import sys
        for key in beacon.keys():
            if '"' in beacon[key]: beacon[key] = beacon[key].replace('"', "\\\"")
        return True

    def close(self):
        self.mysql.commit()
        self.mysql.close()
        self.logging.append("Wrote " + str(self.stats) + " rows to MySQL.")
        self.logging.append("DB closed.")
        self.logging.close()
        return True

def version():
    return "sqllogger.py: 1.6"



from color4console.style import *

def color(text, type = None):
    #Symbols:
    hook = FGCOLOR.NGREEN.value + Symbol.CHECK.value + CEND + TAB
    info = FGCOLOR.NBLUE.value + Symbol.IOTA.value + CEND + TAB
    error = FGCOLOR.NRED.value + Symbol.CHI.value + CEND + TAB

    #Colors:
    red = FGCOLOR.NRED.value
    green = FGCOLOR.NGREEN.value
    yellow = FGCOLOR.NYELLOW.value
    blue = FGCOLOR.NBLUE.value

    if type == "success": style = hook + green
    elif type == "info": style = info + blue
    elif type == "error": style = error + red
    print(style + text + "\033[0m")

def version():
    return 0.1

# This Code is basically just an adapter for the color4console library and is 
# therefore licenced the same way as the color4console library under the MIT Licence.
# All credits go to the original author of the color4console library.

# MIT License

# Copyright (c) 2022 Mahmud Iftekhar Zamil

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


#Copyright (C)2022  Elias Kremer, full license (GNU AGPLv3) in LICENSE.txt
from .errorhandlers import *
from time import ctime as ct

class Logger:
    def __init__(self, config_obj, path, name, short = "LOGG"):
        self.short = short

        runmode = config_obj.getconfig("logmode")
        if runmode == "console":
            from sys import stderr
            self.out = stderr
        elif runmode == "logfile":
            from os.path import exists
            path += "/logs/"
            if not exists(path):
                from os import mkdir
                mkdir(path)
            self.out = open(path + name, "a")
        else:
            self.out = open("/dev/null", "a")

    def append(self, message):
        self.out.write(ct()[4:-5] + ": " + self.short + ": " + str(message) + "\n")
        self.out.flush()
        return True

    def close(self):
        self.out.flush()
        self.out.close()
        return True

def version():
    return 1.2 #Slightly modified version of 1.2



from .colors import color

class SimpleArgParser:
    #option1 = {"Debug mode": [["+d", "enable_debug", "debug_on", 1], ["-d", "disable_debug", "debug_off", 1]]}
    #option2 = {"Django mode": [["+s", "enable_django", "django_on", 1], ["-s", "disable_django", "django_off", 1]]}
    #option3 = {"Helping mode": [["h", "help", "hilb", "halp"]]}
    #Array must contain 4 items. Fill up with a character that doesn't make sense such as 1 if your args are text
    #option1.values()[0] is enable, option1.values()[1] is disable

    def __init__(self, option1, option2 = {"":[["","","",""],["","","",""],False]}, option3 = {"":[["","","",""],["","","",""],False]}):
        global name1, name2, name3, enopt1, disopt1, enopt2, disopt2, enopt3, disopt3, def1, def2, def3
        values1 = list(option1.values())[0]
        values2 = list(option2.values())[0]
        values3 = list(option3.values())[0]

        name1 = list(option1.keys())[0]
        enopt1 = values1[0]
        disopt1 = values1[1]
        def1 = values1[2]

        if option2 != None:
            name2 = list(option2.keys())[0]
            enopt2 = values2[0]
            disopt2 = values2[1]
            def2 = values2[2]
        else:
            name2 = None
            enopt2 = None
            disopt2 = None
            def2 = False

        if option3 != None:
            name3 = list(option3.keys())[0]
            enopt3 = values3[0]
            disopt3 = values3[1]
            def3 = values3[2]
        else:
            name3 = None
            enopt3 = None
            disopt3 = None
            def3 = False


    def setargs(self, args):
        one = def1
        two = def2
        three = def3

        for arg in args:
                if arg == enopt1[0] or arg == enopt1[1] or arg == enopt1[2] or arg == enopt1[3]: 
                    color(f'{name1} enabled.', "info")
                    one = True
                elif arg == disopt1[0] or arg == disopt1[1] or arg == disopt1[2] or arg == disopt1[3]: 
                    color(f'{name1} disabled.', "info")
                    one = False
                elif arg == enopt2[0] or arg == enopt2[1] or arg == enopt2[2] or arg == enopt2[3]: 
                    color(f'{name2} enabled.', "info")
                    two = True
                elif arg == disopt2[0] or arg == disopt2[1] or arg == disopt2[2] or arg == disopt2[3]: 
                    color(f'{name2} disabled.', "info")
                    two = False
                elif arg == enopt3[0] or arg == enopt3[1] or arg == enopt3[2] or arg == enopt3[3]: 
                    color(f'{name3} enabled.', "info")
                    three = True
                elif arg == disopt3[0] or arg == disopt3[1] or arg == disopt3[2] or arg == disopt3[3]: 
                    color(f'{name3} disabled.', "info")
                    three = False
                elif arg == "default":
                    color("Default mode", "info")
                    one = def1
                    two = def2
                    three = def3
                else:
                    color("Invalid argument", "error")
                    raise ValueError("Invalid argument: " + arg) 
        return (one, two, three)
        
    def parse(self, argv, expected_args = 3):
        match len(argv):
            case 1 : one, two, three = self.setargs(["default"])
            case 2: one, two, three = self.setargs([argv[1]])
            case 3: one, two, three = self.setargs([argv[1], argv[2]])
            case 4: one, two, three = self.setargs([argv[1], argv[2], argv[3]])
            case _: color("Too many arguments", "error"); raise ValueError("Too many arguments: got " + str(len(argv) - 1) + " fromm max 3")

        match expected_args:
            case 1: return one
            case 2: return (one, two)
            case 3: return (one, two, three)
            case _: color("Invalid number of expected arguments", "error"); raise ValueError("Invalid number of expected arguments: got " + str(len(argv)) + " fromm max 3")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="elixs",
    version="0.1.3",                               # The release version
    author="Elias Kremer",                         # Full name of the author
    author_email="eliservices.server@gmail.com",   # Email address of the author
    description="Standart Library for elixs.dev",
    long_description=long_description,             # Long description read from the the readme file
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),           # List of all python modules to be installed
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: POSIX :: Linux",
    ],                                             # Information to filter the project on PyPi website
    python_requires='>=3.10',                      # Minimum version requirement of the package
    py_modules=["elixs"],                          # Name of the python package
    package_dir={"elixs": "elixs"},                         # Directory of the source code of the package
    install_requires=["mysql-connector-python", "color4console"]    # Install other dependencies if any
)
